# defaults
desc.globals <- new.env();
# labels used as names on x-axis
desc.globals$plot_labels <- NULL;
# order of the groups in the plots
desc.globals$plot_order <-  NULL;
# colors of the groups in the plots
desc.globals$plot_colors <-  NULL;
# shape of the fonts used when printing the labels
desc.globals$plot_font <-  NULL;
# description files already read?
desc.globals$read <- FALSE;


#' Read a description file.
#'
#' This function reads a description file.
#'
#' This function reads a description file to set up meta data relevant for statistical analyses and vizualisation.
#' The function expects a file with the name “data_description.csv” in the output folder. It will automatically be
#' created with default settings (unless a “data_description.csv” file exist already in the output folder).
#' For more information on the content of the data description file @seealso \code{\link{createDescriptionFileTemplate}}.
#'
#'
#' @param filenamen name of the file containing the meta information (data description). Default is \code{data_description.csv}.
#'
#' @export
readDescriptionFile <- function(filename="data_description.csv"){

  dfile <- paste0(pkg.globals$folder.results,pkg.globals$fsep,filename);
  rl <- NULL;
  if(file.exists(dfile)){
    dmat <- read.csv(dfile,header=TRUE,row.names=1,stringsAsFactors=FALSE);
    dirs2analyse <- rownames(dmat);
    if(sum(dirs2analyse %in% pkg.globals$dirs) != length(dirs2analyse)){
      stop(paste0("Rownames in your ",filename," do not match dirs!"));
    }else{
      pkg.globals$dirs <- dirs2analyse;
    }
    desc.globals$plot_labels <- dmat[,"plot_labels"];
    names(desc.globals$plot_labels) <- rownames(dmat);
    desc.globals$ plot_order <- dmat[,"plot_order"];
    names(desc.globals$plot_order) <- rownames(dmat);
    desc.globals$plot_colors <- dmat[,"plot_colors"];
    names(desc.globals$plot_colors) <- rownames(dmat);
    desc.globals$plot_font <- dmat[,"plot_font"];
    names(desc.globals$plot_font) <- rownames(dmat);
    #rl <- list(plot_labels=plot_labels,plot_order=plot_order,plot_colors=plot_colors,plot_font=plot_font)

    desc.globals$read <- TRUE;

  }else{
    stop(paste0(filename," does not exists. Please create a description file using createDescriptionFileTemplate()."));
  }

}


#' Create a description file.
#'
#' This function creates a description file.
#'
#' This function creates a .csv file containing nessecary information for statistical analyses and vizualisation. The content is organized in a tabular.
#' It has as many rows as folders in the chosen data directory and five columns.
#' \tabular{ccccc}{
#'   \tab plot_labels \tab plot_order \tab plot_colors \tab plot_font \cr
#'   folder 1 \tab folder 1 \tab 1 \tab #FF0000FF \tab 1 \cr
#'   folder 2 \tab folder 2 \tab 2 \tab #00FF00FF \tab 1 \cr
#'   folder 3 \tab folder 3 \tab 3 \tab #0000FFFF \tab 1 \cr
#'   ... \tab ... \tab ... \tab ... \tab ...
#' }
#' The columns contain default definitions that are used for producing plots, e.g. plot labels, plot order, colors or font shape (e.g. normal (1), italic (2) or bold (3)).
#' The entries in the last four columns can be customized by the user. Also, complete rows can be deleted if a certain folder (species, time point or treatment) should be excluded from analysis.
#' The names in the first column have to correspond to the names of the data folders and should not be changed.

#'
#' @param filename name of the file containing the meta information (data description). Default is \code{data_description.csv}.
#'
#'
#' @export
createDescriptionFileTemplate <- function(filename="data_description.csv"){
  dfile <- paste0(pkg.globals$folder.results,pkg.globals$fsep,filename)
  if(file.exists(dfile)){
    stop(paste0(dfile," already exists."))
  }
  dmat <- matrix(NA,nrow=length(pkg.globals$dirs),ncol=4)
  rownames(dmat) <- pkg.globals$dirs
  colnames(dmat) <- c("plot_labels","plot_order","plot_colors","plot_font")
  dmat[,1] <- pkg.globals$dirs
  dmat[,2] <- 1:length(pkg.globals$dirs)
  dmat[,3] <- rainbow(length(pkg.globals$dirs))
  dmat[,4] <- 1
  write.csv(dmat,dfile,row.names=TRUE)

}
