# defaults
feature.globals <- new.env();
# data frame containing the meta information of the features
feature.globals$fmap <- NULL;
# list of features split by their classes (global, contour-based, skeleton-based, pavement cell shape-based)
feature.globals$feat <- NULL;
# total number of features
feature.globals$num_features <- NA;
# default unit
feature.globals$unit <- "microns";

# Read the feature map.
#
# This function reads the feature map included in the package. The map contains the name and class of the features, their order in the feature table and unit.
#
# for internal use only
readFeatureMap <- function(){
  feature.globals$fmap <- read.csv(file=system.file("extdata","feature_map.csv",package="PaCeQuantAna",mustWork=TRUE),header=TRUE,stringsAsFactors=FALSE,comment.char="#",check.names = FALSE)
	rownames(feature.globals$fmap) <- feature.globals$fmap[,1]
	# reorganizing the features by group
	feature.globals$feat <- split(feature.globals$fmap,as.factor(feature.globals$fmap[,"group"]))
	for(j in 1:length(feature.globals$feat)){
  		o <- order(feature.globals$feat[[j]][,"order_group"])
  		feature.globals$feat[[j]] <- feature.globals$feat[[j]][o,]
	}

	# number of features
	feature.globals$num_features <- nrow(feature.globals$fmap);
}



#' @title  Read feature data.
#'
#' @description  This function reads the feature values computed from Pavement cell shapes by PaCeQuant.
#'
#' @details  This function will read all \code{*-table.txt} files recursively from the data directory (and folders specified in the \code{data description} file).
#' Each feature table file corresponds to an image and each line within a feature table corresponds to a cell.
#' For each cell the corresponding image and region IDs, the name of the folder (which corresponds to the condition e.g. species) and the feature values are stored in a list object. Names of the list elements will have the prefix \code{raw}.
#'
#' @return A list containing the following components:
#' \describe{
#'     \item{raw_species}{A vector containing the names of the corresponding folder (e.g. species) of each cell.}
#'     \item{raw_pc.mat}{A matrix containing the feature values of all cells.}
#'     \item{raw_rois}{A vector containing the image and region IDs of all cells.}
#' }
#'
#' @export
readFeatureData <- function(){
  readFeatureMap();

  if(!desc.globals$read){
    stop("No description file was read. Please read a description file using readDescriptionFile(filename=\"data_description_file.csv\")")
  }

  ### NOCH CHECKEN, OB FEATURE MAP und FEATURE DATEN ZUSAMMEN PASSEN !!!

	species <- vector(length=0)
	rois <- vector(length=0)
	pc.mat <- matrix(NA,nrow=0,ncol=feature.globals$num_features)
	for(dir in pkg.globals$dirs){
		#print(dir)
		if(sum(grepl("results",list.dirs(pkg.globals$dir.path,full.names=FALSE))) == 0){
			stop("No \"results\" folders were found!")
		}
		#list.files(dir.path,pattern="-table.txt",recursive=TRUE)
		files <- list.files(path=paste0(pkg.globals$dir.path,pkg.globals$fsep,dir,pkg.globals$fsep,"results"),"-table.txt",full.names=TRUE)
		files.short <- list.files(path=paste0(pkg.globals$dir.path,pkg.globals$fsep,dir,pkg.globals$fsep,"results"),"-table.txt",full.names=FALSE)

		for(j in 1:length(files)){
			f <- files[j]
			data <- read.csv(f,sep="\t",stringsAsFactors=FALSE,check.names = FALSE)

			# noch feature namen ueberpruefen!!!

			N <- nrow(data)
			species <- c(species,rep(dir,N))

			file <- strsplit(files.short[j],"-table.txt")[[1]][1]
			# rois fangen bei 1 an zu zaehlen
			rois <- c(rois,paste(file,"-roi_",data[,1],sep=""))

			pc.mat <- rbind(pc.mat,data[,-1])
		}
	}
	return(list(raw_species=species,raw_pc.mat=pc.mat,raw_rois=rois))

}

#' @title Print some statistics on pavement cells.
#'
#' @description This function will print the number of images per group, cells per group, and cells per image and group.
#'
#' @param data feature data containing list generated by \code{\link{readFeatureData}}.
#' @param type defines the subset of cells. Can either be \code{raw}, \code{filtered} or \code{small}. Default is \code{raw}.
#' @param printToFile if \code{printToFile=FALSE}, all information is printed to the console. If \code{printToFile=TRUE} information is printed to a file saved to the \code{stats} folder in the output folder (\code{\link{setOutputFolder}}).
#'
#' @export
#' @examples
#' data <- readFeatureData()
#' printStatsOnCells(data)
printStatsOnCells <- function(data=data,type="raw",printToFile=FALSE){
  if(!(type %in% c("raw","filtered","small"))){
    stop("Entered type must either be \"raw\" or \"filtered\".");
  }

  if(!desc.globals$read){
    stop("No description file was read. Please read a description file using readDescriptionFile(filename=\"data_description_file.csv\")")
  }

  if(type == "raw"){
    if(sum(grepl("raw",names(data))) == 0){
      stop("No raw data available. You need to run \"readFeatureData()\".")
    }else{
      species <- data$raw_species;
      rois <- data$raw_rois;
      pc.mat <- data$raw_pc.mat;
    }
  }else if(type == "filtered"){
    if(sum(grepl("filtered",names(data))) == 0){
      stop("No filtered data available. You need to apply \"filterOutSmallCells(data)\".")
    }else{
      species <- data$filtered_species;
      rois <- data$filtered_rois;
      pc.mat <- data$filtered_pc.mat;
    }
  }else{
    if(sum(grepl("small",names(data))) == 0){
      stop("No small data available. You need to apply \"filterOutSmallCells(data)\".")
    }else{
      species <- data$small_species;
      rois <- data$small_rois;
      pc.mat <- data$small_pc.mat;
    }
  }
  uspecies <- unique(species)
  sf <- factor(species,levels=uspecies)
  da <- split(pc.mat[,1],sf)
  if(printToFile){
    ofile <- paste0(pkg.globals$folder.results,pkg.globals$fsep,"stats",pkg.globals$fsep,paste0("statsOnCells_",type,".txt"));
    write(paste0("stats on cells: ",type),file=ofile);
    write("",file=ofile,append=TRUE)
    # number of images
    write("Number of images:",file=ofile,append=TRUE)
    tmp <- lapply(split(rois,sf),function(el){length(unique(sapply(el,function(el2){strsplit(el2,"-roi_")[[1]][1]})))})
    for(j in 1:length(tmp)){
      write(paste0(names(tmp)[j],": ",tmp[j]),file=ofile,append=TRUE)
    }
    write("",file=ofile,append=TRUE)
    # number of cells
    write("Number of cells: ",file=ofile,append=TRUE)
    tmp <- lapply(da,length);
    for(j in 1:length(tmp)){
      write(paste0(names(tmp)[j],": ",tmp[j]),file=ofile,append=TRUE)
    }
    write("",file=ofile,append=TRUE)
    # number of cells per image
    write("Number of cells per image:",file=ofile,append=TRUE)
    tmp <- lapply(split(rois,sf),function(el){table(sapply(el,function(el2){strsplit(el2,"-roi_")[[1]][1]}))});
    for(j in 1:length(tmp)){
      write(paste0(names(tmp)[j],": "),file=ofile,append=TRUE)
      write(paste(names(tmp[[j]]),collapse="\t"),file=ofile,append=TRUE)
      #print(names(tmp[[j]]))
      write(paste(unlist(tmp[j]),collapse="\t"),file=ofile,append=TRUE)
    }
  }else{
    # number of images
    print("Number of images:")
    print(lapply(split(rois,sf),function(el){length(unique(sapply(el,function(el2){strsplit(el2,"-roi_")[[1]][1]})))}))
    # number of cells
    print("Number of cells: ")
    print(lapply(da,length))

    # number of cells per image
    print("Number of cells per image:")
    print(lapply(split(rois,sf),function(el){table(sapply(el,function(el2){strsplit(el2,"-roi_")[[1]][1]}))}))
  }
}


#' @title Filter out small cells.
#'
#' @description This function filters out cells with an area smaller than defined by \code{\link{setSizeThresholds}}.
#'
#' @param data feature data containing list generated by \code{\link{readFeatureData}}.
#'
#' @return an extended version of the data containing list generated by \code{\link{readFeatureData}}. Names of the new list elements will have the prefix \code{filtered} and \code{small}.
#' @export
#'
#' @examples
#' data <- readFeatureData()
#' data <- filterOutSmallCells(data)
filterOutSmallCells <- function(data){
  if(sum(grepl("raw",names(data))) == 0){
    stop("No raw data available. You need to run \"readFeatureData()\".")
  }else{
    species <- data$raw_species;
    rois <- data$raw_rois;
    pc.mat <- data$raw_pc.mat;
  }

  if(sum(grepl("filtered",names(data))) >0){
    print("data already contained filtered data! These are replaced by this function call!")
  }

  data2 <- vector("list",length=9);
  data2$raw_species <- data$raw_species;
  data2$raw_rois <- data$raw_rois;
  data2$raw_pc.mat <- data$raw_pc.mat;

  global.th <- unlist(getSizeThresholds())
  idx.f <- pc.mat[,1] >= global.th["small"]
  idx.s <- pc.mat[,1] < global.th["small"]
  pc.mat.f <- pc.mat[idx.f,]
  pc.mat.s <- pc.mat[idx.s,]
  species.f <- species[idx.f]
  species.s <- species[idx.s]
  rois.f <- rois[idx.f]
  rois.s <- rois[idx.s]

  data2$filtered_species <- species.f
  data2$filtered_pc.mat <- pc.mat.f
  data2$filtered_rois <- rois.f

  data2$small_species <- species.s
  data2$small_pc.mat <- pc.mat.s
  data2$small_rois <- rois.s

  return(data2)
}

#' @title Set the unit used in PaCeQuant for processing of images.
#'
#' @description This function can be used to set or change the unit that was selected in PaCeQuant for image processing.
#'
#' @param unit can either be \code{microns} or \code{pixel}. Default is \code{microns}.
#'
#' @export
setUnit <- function(unit="microns"){
  if(!(unit %in% c("microns","pixel"))){
    stop("Entered unit must either be \"microns\" or \"pixel\".");
  }else{
    feature.globals$unit <- unit;
  }
}
