# defaults
pkg.globals <- new.env()
# output folder
pkg.globals$folder.results <- "output"
# path to data directory
pkg.globals$dir.path <- NA;
# names of the folders contained in the data directory
pkg.globals$dirs <- NA;
# platform-specific seperator "/" or "\"
pkg.globals$fsep <-.Platform$file.sep

# checks if a folder already exists
#
# for internal use only
checkCreateFolder <- function(folder){
  if(dir.exists(folder)){
    print(paste0(folder," already exists."))
  }else{
    dir.create(folder)
  }
}

# checks if a data type raw, filtered or small exists
# additionally checks if a description file was read
#
# for internal use only
checkType <- function(data,type){
  if(!desc.globals$read){
    stop("No description file was read. Please read a description file using readDescriptionFile(filename=\"data_description_file.csv\")")
  }

  if(type == "raw"){
    if(sum(grepl("raw",names(data))) == 0){
      stop("No raw data available. You need to run \"readFeatureData()\".")
    }else{
      species <- data$raw_species;
      rois <- data$raw_rois;
      pc.mat <- data$raw_pc.mat;
    }
  }else if(type == "filtered"){
    if(sum(grepl("filtered",names(data))) == 0){
      stop("No filtered data available. You need to apply \"filterOutSmallCells(data)\".")
    }else{
      species <- data$filtered_species;
      rois <- data$filtered_rois;
      pc.mat <- data$filtered_pc.mat;
    }
  }else{
    if(sum(grepl("small",names(data))) == 0){
      stop("No small data available. You need to apply \"filterOutSmallCells(data)\".")
    }else{
      species <- data$small_species;
      rois <- data$small_rois;
      pc.mat <- data$small_pc.mat;
    }
  }
  return(list(species=species,pc.mat=pc.mat,rois=rois))
}


#' @title Set output folder.
#'
#' @description Set the name of the folder that is created in the current working directory. All output files that are generated in the following workflows are stored there.
#'
#' @details This functions additionally creates the following folders in the selected output folder:
#' \describe{
#'    \item{histograms}{@seealso \code{\link{plotHistogramOnArea}} and \code{\link{plotHistograms}}}
#'    \item{box_plots}{@seealso \code{\link{plotBoxplots}}}
#'    \item{violin_plots}{@seealso \code{\link{plotViolinplots}}}
#'    \item{scatter_plots}{@seealso \code{\link{plotScatterplots}}}
#'    \item{cumulative_distributions}{@seealso \code{\code{plotCumulativeDistributions}}}
#'    \item{kw_dunn}{@seealso \code{\link{kwDunnTest}}}
#'    \item{stats}{@seealso \code{\link{printStatsOnCells}}}
#' }
#'
#' @param name name of the output folder. Default is \code{output}.
#'
#' @export
setOutputFolder <- function(name="output"){
  pkg.globals$folder.results <- name;

  # folder.results
  checkCreateFolder(pkg.globals$folder.results)
  # box_plots
  checkCreateFolder(paste0(pkg.globals$folder.results,pkg.globals$fsep,"box_plots"))
  # violin_plots
  checkCreateFolder(paste0(pkg.globals$folder.results,pkg.globals$fsep,"violin_plots"))
  # scatter_plots
  checkCreateFolder(paste0(pkg.globals$folder.results,pkg.globals$fsep,"scatter_plots"))
  # histograms
  checkCreateFolder(paste0(pkg.globals$folder.results,pkg.globals$fsep,"histograms"))
  # cummulative_distributions
  checkCreateFolder(paste0(pkg.globals$folder.results,pkg.globals$fsep,"cumulative_distributions"))
  # kw_dunn
  checkCreateFolder(paste0(pkg.globals$folder.results,pkg.globals$fsep,"kw_dunn"))
  # stats
  checkCreateFolder(paste0(pkg.globals$folder.results,pkg.globals$fsep,"stats"))
}


#' @title Set path to data directory.
#'
#' @description Sets the path to the directory that contains the PaCeQuant output.
#'
#' @details All folders in this directory will be included in the analyses by default. Selections of specific folders can be specified in the \code{data description} file @seealso \code{\link{readDescriptionFile}}. Each folder will be handled as an individual group.
#'
#' @param name path to the data containing directory. Default is the current working directory (\code{getwd()}).
#'
#' @export
setDataDir <- function(name=getwd()){
  pkg.globals$dir.path <- name;
	# check if directory exists
  pkg.globals$dirs <- list.files(path=pkg.globals$dir.path,full.names=FALSE);
}


