
######



#' @title Plot histograms of the cell areas.
#'
#' @description This function plots histograms of the measured areas of pavement cells. Plots are saved to the \code{histograms} folder in the \code{output} folder (@seealso \code{\link{setOutputFolder}}).
#'
#' @param data feature data containing list generated by \code{\link{readFeatureData}} and \code{\link{filterOutSmallCells}}.
#' @param type of the data can be
#' \describe{
#'     \item{raw}{all measured cell areas. Default.}
#'     \item{filtered}{cell areas above a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#'     \item{small}{cell areas below a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#' }
#' @param show_ts threshold for small cells (@seealso \code{\link{setSizeThresholds}}). Default is \code{show_ts=TRUE}.
#' @param show_tm threshold for medium size cells (@seealso \code{\link{setSizeThresholds}}). Default is \code{show_tm=TRUE}.
#' @param alpha transparency value used in RGB color space. Default is \code{alpha=0.5}.
#' @param fill \code{TRUE} or \code{FALSE} depending on whether the area under the histgram curve should be filled or not. Default is \code{fill=TRUE}.
#' @param legend detailed information on the plot
#' \describe{
#'     \item{0}{no legend. Default.}
#'     \item{1}{minimal legend in the right upper corner of the plot}
#'     \item{2}{extended legend in the right upper corner of the plot with additional information on the number of cell}
#' }
#' @param bs size of the bins. Default is \code{bs=200}.
#' @param max.ul the maximum cell size or max value to be plotted on the x-axis. If it is set to \code{NA} then the value is automatically determinded from the data. Default is \code{max.ul=NA}.
#'
#' @export
plotHistogramOnArea <- function(data=data, type="raw", show_ts=TRUE, show_tm=TRUE,alpha=0.5,fill=TRUE,legend=0,bs=200,max.ul=NA){
#area,species,species.raw,outdir,base_file_name,col.hist,alpha=0.5,area.th = NA,fill=TRUE,legend=0,ncol_legend=1,bs=200,max.ul=NA
  outdir <- paste0(pkg.globals$folder.results,pkg.globals$fsep,"histograms");
  base_file_name <- paste0(type,"_data");

  tmpf <- checkType(data,type);
  species <- tmpf$species;
  pc.mat <- tmpf$pc.mat;
  rois <- tmpf$rois;

  species.raw <- NULL;

  if(type == "filtered"){
    if(sum(grepl("filtered",names(data))) == 0){
      stop("No filtered data available. You need to apply \"filterOutSmallCells(data)\".")
    }else{
      species.raw <- data$filtered_species;
    }
  }else if(type == "raw"){
    if(sum(grepl("filtered",names(data))) > 0){
      species.raw <- data$filtered_species;
    }
  }

  area <- pc.mat[,1]
  area.th <- unlist(getSizeThresholds())
  K <- length(area.th);
  k.names <- names(area.th)
#  print(k.names)
  plot_labels <- desc.globals$plot_labels[order(desc.globals$plot_order)]
  plot_font <- desc.globals$plot_font
  col.hist <- desc.globals$plot_colors
  names(col.hist) <- unique(species)

  col.hist <- col.hist[order(desc.globals$plot_order)]
  #uspecies <- unique(species)
  uspecies <- names(plot_labels)

  sf <- factor(species,levels=uspecies)
  rg <- range(area)
  if(is.na(max.ul)){
    ul <- ceiling(max(rg))
  }else{
    ul <- max.ul
  }
#  print(ul)
#  print(bs)
  breaks <- seq(bs,ul,by=bs)
  breaks <- c(floor(rg[1]),breaks,breaks[length(breaks)]+bs)
  #if(max(breaks) < ul){
  #	breaks <- c(breaks,ceiling(ul))
  #}
  #print(breaks)
  da <- split(area,sf)
  n <- table(species)[names(plot_labels)]

  if(!is.null(species.raw)){
    n.r <- table(species.raw)[names(plot_labels)]
  }

  real.yl <- max(unlist(lapply(da,function(el){h <- hist(el,breaks=breaks,plot=FALSE);max(h$counts/sum(h$counts,na.rm=TRUE))})))
  yl <- 0.2
  if(real.yl > yl){
    yl <- ceiling(real.yl*100)/100
  }
#  print(yl)

  fnum <- length(list.files(path = outdir,pattern = paste0(base_file_name,"_relative_histogram_area_bs",bs)))
  if(fnum == 0){
    ofile <- paste0(outdir,pkg.globals$fsep,base_file_name,"_relative_histogram_area_bs",bs,".pdf");
  }else{
    ofile <- paste0(outdir,pkg.globals$fsep,base_file_name,"_relative_histogram_area_bs",bs,"_",fnum,".pdf");
  }
  pdf(file=ofile,width=8,height=2.5,family="Arial",useDingbats=FALSE)
  par(mgp=c(2.2,1,0),mai=c(0.65,0.8,0.5,0.2))
  plot(0,0,col=0,xlim=c(bs,ul),ylim=c(0,yl),ylab="Relative counts",xlab=expression("Area ["*mu*m^2*"]"),main="",axes=FALSE)
  box()
  axis(2)
  axis(1,seq(0,ceiling(ul/1000)*1000,by=1000))

  if(show_ts || show_tm){

    if(length(area.th) == 1){
      if(show_ts){
        abline(v=area.th,col=1,lty=2)
        thn <- substr(k.names[1],1,1)
        #mtext(expression(t[k]),at=area.th[k],side=3)
        mtext(substitute(t[s],list(s=thn)),at=area.th[1],side=3)
      }
    }else{
      for(k in 1:(length(area.th)-1)){
        anno <- FALSE
        if(k == 1 & show_ts){
          anno <- TRUE
        }
        if(k == 2 & show_tm){
          anno <- TRUE
        }

        if(anno){
          abline(v=area.th[k],col=1,lty=2)
          thn <- substr(k.names[k],1,1)
          #mtext(expression(t[k]),at=area.th[k],side=3)
          mtext(substitute(t[s],list(s=thn)),at=area.th[k],side=3)
        }
      }
      if(length(area.th) > 1){
        if(show_tm){
          text(mean(c(0,area.th[1])),yl-0.01,labels=k.names[1])
          for(j in 2:length(area.th)){
            text(mean(area.th[(j-1):(j)]),yl-0.01,labels=k.names[j])
          }
          text(mean(c(area.th[length(area.th)-1],ul)),yl-0.01,labels=k.names[K])
        }
      }
    }

  }

  col.trans <- col.hist
  #uspecies <- unique(species)
  for(j in 1:(length(uspecies))){
#    print(uspecies[j])
#    print(range(da[[j]]))
    h <- hist(da[[j]],breaks=breaks,plot=FALSE)
    #idxp <- h$counts != 0
    #xx <- h$mids[idxp]
    #yy <- h$counts[idxp]/n[uspecies[j]]

    xx <- h$mids
    yy <- h$counts/n[uspecies[j]]

    if(fill){
      colpo <- col2rgb(col.hist[j])/255
      col.trans[j] <- rgb(colpo[1],colpo[2],colpo[3],alpha=alpha)
      polygon(c(xx,rev(xx)),c(yy,rep(0,length(yy))),border = NA,col=col.trans[j])
    }
    lines(xx,yy,col=col.hist[j],lwd=2)
  }
  #legend("topright",legend=paste(uspecies," (",n," cells)",sep=""),fill=col.hist,box.lty=0)
  #leg.tmp <- legend(x=ul-1000,y=yl,legend=plot_labels,fill=col.trans,border=col.hist,box.lty=0)
  if(legend %in% c(1,2)){
    ladj <- 10;
    if(legend == 1){
      shift <- ceiling(max(strwidth(plot_labels))+max(strwidth(paste0("(N=", n,")")))+ strwidth(" "))
#      print(shift)
      if(fill){
        leg.tmp <- legend(x=ul-shift-(0.2*shift),y=yl+yl/ladj,legend=plot_labels,fill=col.hist,box.lty=0,text.font=plot_font[names(plot_labels)])
      }else{
        leg.tmp <- legend(x=ul-shift-(0.3*shift),y=yl+yl/ladj,legend=plot_labels,col=col.hist,lty=1,box.lty=0,lwd=2,text.font=plot_font[names(plot_labels)])
      }
      text(leg.tmp$text$x +max(strwidth(plot_labels)) + strwidth(" "), leg.tmp$text$y,paste0("(N=", n,")"), font=1, adj=c(0,.5))
    }else{
#      print(ul)
      shift <- ceiling(max(strwidth(plot_labels))+max(strwidth(substitute(paste("(",N[total],"=", n,", N=",n.r ,")"),list(n=n,n.r=n.r)))))
#      print(shift)
      if(fill){
        leg.tmp <- legend(x=ul-shift-(0.1*shift),y=yl+yl/ladj,legend=plot_labels,fill=col.hist,box.lty=0,text.font=plot_font[names(plot_labels)])
      }else{
        leg.tmp <- legend(x=ul-shift-(0.1*shift),y=yl+yl/ladj,legend=plot_labels,col=col.hist,lty=1,box.lty=0,lwd=2,text.font=plot_font[names(plot_labels)])
      }
      max.strwidth <- max(strwidth(plot_labels))
      for(j in 1:length(plot_labels)){
        text(leg.tmp$text$x + max.strwidth+ strwidth(" "), leg.tmp$text$y[j],substitute(paste("(",N[total],"=", n,", N=",n.r ,")"),list(n=n[j],n.r=n.r[j])), font=1, adj=c(0,.5),cex=0.8)
      }
    }
  }

  dev.off()


}

######
# @title Parse the label of a feature unit.
#
# @description This function parses the label of a feature unit as provided in the data description file.
#
# @param unit feature unit
# @param lab label to be plotted as axis label
#
# @return axis label
# for internal use only
parse.unit.xlab <- function(unit, lab){
  Runit <- as.character(unit)
  ### TODO check unit: feature.globals$unit

  if(Runit == "microns^2"){
    unit <- expression(mu*m^2)
    #unit <- "mu*m^2"
    lleft <- paste0(lab," [")
    lright <- "]"
  }else if(Runit == "microns"){
    unit <- expression(mu*m)
    lleft <- paste0(lab," [")
    lright <- "]"
  }else if(Runit == "pixel^2"){
    unit <- expression(px^2)
    #unit <- "mu*m^2"
    lleft <- paste0(lab," [")
    lright <- "]"
  }else if(Runit == "pixel"){
    unit <- expression(px)
    lleft <- paste0(lab," [")
    lright <- "]"
  }else{
    unit <- expression("")
    lleft <- lab
    lright <- ""
  }
  xlab <- eval(bquote(expression(.(lleft) * .(unit[[1]]) * .(lright))))
  return(xlab)
}

######

#' @title Plot histograms of feature values.
#'
#' @description This function plots histograms of all features where the histogram of each feature is plotted on a separate page of a pdf file. Plots are saved to the \code{histograms} folder in the \code{output} folder (@seealso \code{\link{setOutputFolder}}).
#'
#' @param data feature data containing list generated by \code{\link{readFeatureData}} and \code{\link{filterOutSmallCells}}.
#' @param type of the data can be
#' \describe{
#'     \item{raw}{all measured cell areas. Default.}
#'     \item{filtered}{cell areas above a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#'     \item{small}{cell areas below a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#' }
#' @param fill \code{TRUE} or \code{FALSE} depending on whether the area under the histgram curve should be filled or not. Default is \code{fill=TRUE}.
#' @param legend \code{TRUE} or \code{FALSE} depending on whether the legend should be plotted in the upper right corner of the plot or once on a separate page. Default is \code{legend=TRUE}.
#' @param ncol_legend number of columns to arrange to content of the legend. Default is \code{ncol_legend=1}.
#'
#' @export
plotHistograms <- function(data=data,type="raw",fill=TRUE,legend=TRUE,ncol_legend=1){

  outdir <- paste0(pkg.globals$folder.results,pkg.globals$fsep,"histograms");
  base_file_name <- paste0(type,"_data");

  fsep <- pkg.globals$fsep;
  features <- feature.globals$feat;

  tmpf <- checkType(data,type);
  species <- tmpf$species;
  pc.mat <- tmpf$pc.mat;
  rois <- tmpf$rois;

  plot_labels <- desc.globals$plot_labels[order(desc.globals$plot_order)]
  plot_font <- desc.globals$plot_font
  col.hist <- desc.globals$plot_colors
  names(col.hist) <- unique(species)

  col.hist <- col.hist[order(desc.globals$plot_order)]

  n <- table(species)
  uspecies <- unique(species)
  sf <- factor(species,levels=uspecies)
  col.trans <- col.hist
  for(j in 1:length(uspecies)){
    colpo <- col2rgb(col.hist[j])/255
    col.trans[j] <- rgb(colpo[1],colpo[2],colpo[3],alpha=0.5)
  }

  fnum <- length(list.files(path = outdir,pattern = paste0(base_file_name,"_relative_histograms")))
  if(fnum == 0){
    ofile <- paste0(outdir,fsep,base_file_name,"_relative_histograms.pdf");
  }else{
    ofile <- paste0(outdir,fsep,base_file_name,"_relative_histograms_",fnum,".pdf");
  }

  pdf(file=ofile,width=6,height=2.5,family="Arial",useDingbats=FALSE)
  par(mgp=c(2.2,1,0),mai=c(0.8,0.8,0.5,0.1))
  for(l in 1:length(features)){
    fm <- features[[l]]
    for(k in 1:nrow(fm)){
      j <- fm[k,"order_column"]
      pc <- split(pc.mat[,j],sf)
      mm <- max(unlist(pc),na.rm=TRUE)
      bin <- ceiling(mm/30)
      mmin <- floor(min(unlist(pc),na.rm=TRUE))
      breaks <- seq(0,ceiling(mm),length=30)

      real.yl <- max(unlist(lapply(pc,function(el){h <- hist(el,breaks=breaks,plot=FALSE);max(h$counts/sum(h$counts,na.rm=TRUE))})))

      yl <- ceiling(real.yl*100)/100


      xlab <- parse.unit.xlab(fm[k,"unit"],fm[k,"feature"])
      plot(0,0,col=0,xlim=c(mmin,ceiling(mm)),ylim=c(0,yl),ylab="Relative counts",xlab=xlab,main="",axes=FALSE)
      box()
      axis(2)
      ex <- 3
      if(bin < 6){
        ex <- 1
      }
      axis(1,seq(0,ceiling(mm),by=bin*ex))

      for(a in 1:length(uspecies)){
        h <- hist(pc[[a]],breaks=breaks,plot=FALSE)
        idxp <- h$counts != 0
        xx <- h$mids[idxp]
        yy <- h$counts[idxp]/n[uspecies[a]]
        if(fill){
          polygon(c(xx,rev(xx)),c(yy,rep(0,length(yy))),border = NA,col=col.trans[a])
        }
        lines(xx,yy,col=col.hist[a],lwd=2)
      }
      if(legend){
        legend("topleft",legend=plot_labels,col=col.hist,lty=1,lwd=2,box.lty=0)
      }
    }
  }
  if(!legend){

    ymax <- ceiling(length(plot_labels)/ncol_legend)

    plot(0,0,col=0,ylim=c(0,ymax+1),xlim=c(0,ncol_legend*2),main="",axes=FALSE,xlab="",ylab="")
    start <- 1;
    end <- ymax;
    for(k in 1:ncol_legend){

      legend(x=(k-1)*2,y=ymax,legend=plot_labels[start:end],col=col.hist[start:end],lty=1,lwd=2,box.lty=0,cex=0.8)
      start <- end + 1;
      end <- end + ymax;
      if(end > length(plot_labels)){
        end <- length(plot_labels)
      }
    }

  }
  dev.off()
}

#' @title Plot violin plots of feature values.
#'
#' @description This function plots violin plots of all features where the violins of each feature are plotted on a separate page of a pdf file. Plots are saved to the \code{violin_plots} folder in the \code{output} folder (@seealso \code{\link{setOutputFolder}}).
#'
#' @param data feature data containing list generated by \code{\link{readFeatureData}} and \code{\link{filterOutSmallCells}}.
#' @param type of the data can be
#' \describe{
#'     \item{raw}{all measured cell areas. Default.}
#'     \item{filtered}{cell areas above a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#'     \item{small}{cell areas below a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#' }
#' @param ncol_legend number of columns to arrange to content of the legend. Default is \code{ncol_legend=1}.
#' @param mai_bottom size of the space for the bottom side of the plot to fit the labels (@seealso \code{\link{par}}). Default is \code{mai_bottom=2}.
#' @param mod.h the shape of the violins is determined by the data density. Setting \code{mod.h=TRUE} will re-estimate the density between the 0.05th and 0.95th quantile of the feature values.
#'
plotViolinplots <- function(data=data,type="raw",ncol_legend=1,mai_bottom=2,mod.h=FALSE){

  outdir <- paste0(pkg.globals$folder.results,pkg.globals$fsep,"violin_plots");
  base_file_name <- paste0(type,"_data");

  fsep <- pkg.globals$fsep;
  features <- feature.globals$feat;

  tmpf <- checkType(data,type);
  species <- tmpf$species;
  pc.mat <- tmpf$pc.mat;
  rois <- tmpf$rois;

  plot_order <- desc.globals$plot_order
  plot_labels <- desc.globals$plot_labels[order(plot_order)]
  plot_font <- desc.globals$plot_font
  col.vio <- desc.globals$plot_colors
  names(col.vio) <- unique(species)

  #col.vio <- col.vio[order(desc.globals$plot_order)]

  uspecies <- names(plot_labels)
  sf <- factor(species,levels=uspecies)

  oo <- names(plot_labels)
  col.vio <- col.vio[oo]
  height <- 5.5
  if(mai_bottom != 2){
    height <- height - 2 + mai_bottom
  }
  h <- NULL

  fnum <- length(list.files(path = outdir,pattern = paste0(base_file_name,"_violin_plots")))
  if(fnum == 0){
    ofile <- paste0(outdir,fsep,base_file_name,"_violin_plots.pdf");
  }else{
    ofile <- paste0(outdir,fsep,base_file_name,"_violin_plots_",fnum,".pdf");
  }

  pdf(file=ofile,width=1+length(plot_labels),height=height,family="Arial",useDingbats=FALSE)
  par(mgp=c(2.5,1,0),mai=c(mai_bottom,0.8,0.5,0.2))
  for(l in 1:length(features)){
    fm <- features[[l]]
    for(k in 1:nrow(fm)){
      j <- fm[k,"order_column"]
      pc <- split(pc.mat[,j],sf)

      if(mod.h){
        pc.flat <- unlist(pc)
        q <- quantile(pc.flat,probs=c(0.05,0.95),na.rm=TRUE)
        h <- sm.density(pc.flat[pc.flat > q[1] & pc.flat < q[2]],display="none")$h
      }
      violins(pc[oo],connect=FALSE,col=col.vio[oo],ylab="",names=rep("",length(uspecies)),CImed=FALSE,colMed="white",quantiles=FALSE,deciles=FALSE,main="",h=h)
      title(main=list(parse.unit.xlab(fm[k,"unit"],fm[k,"feature"]),cex=1.5))
      mtext(plot_labels,side=1,at=plot_order[oo],line=1,cex=1,las=2,font=plot_font[oo])
    }
  }
  ymax <- ceiling(length(plot_labels)/ncol_legend)

  plot(0,0,col=0,ylim=c(0,ymax+1),xlim=c(0,ncol_legend*2),main="",axes=FALSE,xlab="",ylab="")
  start <- 1;
  end <- ymax;
  for(k in 1:ncol_legend){


    legend(x=(k-1)*2,y=ymax,legend=plot_labels[start:end],fill=col.vio[start:end],box.lty=0,cex=1)
    start <- end + 1;
    end <- end + ymax;
    if(end > length(plot_labels)){
      end <- length(plot_labels)
    }
  }
  dev.off()
}

#' @title Plot box plots of feature values.
#'
#' @description This function plots box plots of all features where boxes of each feature are plotted on a separate page of a pdf file. Plots are saved to the \code{box_plots} folder in the \code{output} folder (@seealso \code{\link{setOutputFolder}}).
#'
#' @param data feature data containing list generated by \code{\link{readFeatureData}} and \code{\link{filterOutSmallCells}}.
#' @param type of the data can be
#' \describe{
#'     \item{raw}{all measured cell areas. Default.}
#'     \item{filtered}{cell areas above a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#'     \item{small}{cell areas below a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#' }
#' @param ncol_legend number of columns to arrange to content of the legend. Default is \code{ncol_legend=1}.
#' @param mai_bottom size of the space for the bottom side of the plot to fit the labels (@seealso \code{\link{par}}). Default is \code{mai_bottom=2}.
#'
#' @export
#'
plotBoxplots <- function(data=data,type="raw",ncol_legend=1,mai_bottom=2){

  outdir <- paste0(pkg.globals$folder.results,pkg.globals$fsep,"box_plots");
  base_file_name <- paste0(type,"_data");

  fsep <- pkg.globals$fsep;
  features <- feature.globals$feat;

  tmpf <- checkType(data,type);
  species <- tmpf$species;
  pc.mat <- tmpf$pc.mat;
  rois <- tmpf$rois;

  plot_order <- desc.globals$plot_order
  plot_labels <- desc.globals$plot_labels[order(plot_order)]
  plot_font <- desc.globals$plot_font
  col.box <- desc.globals$plot_colors
  names(col.box) <- unique(species)

  uspecies <- names(plot_labels)

  sf <- factor(species,levels=uspecies)

  oo <- names(plot_labels)
  col.box <- col.box[oo]
  height <- 5.5
  if(mai_bottom != 2){
    height <- height - 2 + mai_bottom
  }

  fnum <- length(list.files(path = outdir,pattern = paste0(base_file_name,"_box_plots")))
  if(fnum == 0){
    ofile <- paste0(outdir,fsep,base_file_name,"_box_plots.pdf");
  }else{
    ofile <- paste0(outdir,fsep,base_file_name,"_box_plots_",fnum,".pdf");
  }

  pdf(file=ofile,width=1+length(plot_labels),height=height,family="Arial",useDingbats=FALSE)
  par(mgp=c(2.5,1,0),mai=c(mai_bottom,0.8,0.5,0.2))
  for(l in 1:length(features)){
    fm <- features[[l]]
    for(k in 1:nrow(fm)){
      j <- fm[k,"order_column"]
      pc <- split(pc.mat[,j],sf)
      pp <- pretty(pc.mat[,j],n=4)
      boxplot(pc[oo],col=col.box[oo],ylab="",names=plot_labels,main="",axes=FALSE)
      title(main=list(parse.unit.xlab(fm[k,"unit"],fm[k,"feature"]),cex=1.5))
      box()
      axis(2,at=pp,cex.axis=1.3)
      #axis(1,at=plot_order[oo],labels=plot_labels,cex.axis=1.0,las=2,font=plot_font[oo])
      axis(1,at=plot_order[oo],labels=NA)
      mtext(plot_labels,side=1,at=plot_order[oo],line=1,cex=1,las=2,font=plot_font[oo])
    }
  }
  ymax <- ceiling(length(plot_labels)/ncol_legend)

  plot(0,0,col=0,ylim=c(0,ymax+1),xlim=c(0,ncol_legend*2),main="",axes=FALSE,xlab="",ylab="")
  start <- 1;
  end <- ymax;
  for(k in 1:ncol_legend){

    legend(x=(k-1)*2,y=ymax,legend=plot_labels[start:end],fill=col.box[start:end],box.lty=0,cex=1)
    start <- end + 1;
    end <- end + ymax;
    if(end > length(plot_labels)){
      end <- length(plot_labels)
    }
  }
  dev.off()
}

#' @title Plot cumulative distributions of feature values.
#'
#' @description This function plots cumulative distributions of all features where the cumulative distributions of each feature are plotted on a separate page of a pdf file. Plots are saved to the \code{cumulative_distributions} folder in the \code{output} folder (@seealso \code{\link{setOutputFolder}}).
#'
#' @param data feature data containing list generated by \code{\link{readFeatureData}} and \code{\link{filterOutSmallCells}}.
#' @param type of the data can be
#' \describe{
#'     \item{raw}{all measured cell areas. Default.}
#'     \item{filtered}{cell areas above a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#'     \item{small}{cell areas below a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#' }
#' @param legend \code{TRUE} or \code{FALSE} depending on whether the legend should be plotted in the upper right corner of the plot or once on a separate page. Default is \code{legend=TRUE}.
#' @param ncol_legend number of columns to arrange to content of the legend. Default is \code{ncol_legend=1}.
#'
#' @export
#'
plotCumulativeDistributions <- function(data=data,type="raw",legend=TRUE,ncol_legend=1){


  outdir <- paste0(pkg.globals$folder.results,pkg.globals$fsep,"cumulative_distributions");
  base_file_name <- paste0(type,"_data");

  fsep <- pkg.globals$fsep;
  features <- feature.globals$feat;

  tmpf <- checkType(data,type);
  species <- tmpf$species;
  pc.mat <- tmpf$pc.mat;
  rois <- tmpf$rois;

  plot_labels <- desc.globals$plot_labels[order(desc.globals$plot_order)]
  plot_font <- desc.globals$plot_font[order(desc.globals$plot_order)]
  col.cum <- desc.globals$plot_colors[order(desc.globals$plot_order)]

  uspecies <- pkg.globals$dirs[order(desc.globals$plot_order)]
  sf <- factor(species,levels=uspecies)



  fnum <- length(list.files(path = outdir,pattern = paste0(base_file_name,"_cumulative_distribution_plots")))
  if(fnum == 0){
    ofile <- paste0(outdir,fsep,base_file_name,"_cumulative_distribution_plots.pdf");
  }else{
    ofile <- paste0(outdir,fsep,base_file_name,"_cumulative_distribution_plots_",fnum,".pdf");
  }

  pdf(file=ofile,width=4,height=4,family="Arial",useDingbats=FALSE)
  par(mgp=c(2.5,1,0),mai=c(0.6,0.8,0.5,0.2))
  for(l in 1:length(features)){
    fm <- features[[l]]
    for(k in 1:nrow(fm)){
      j <- fm[k,"order_column"]
      pc <- split(pc.mat[,j],sf)
      pp <- range(pc.mat[,j],na.rm=TRUE)
      plot(0,0,col=0,xlim=pp,ylim=c(0,1),xlab="",ylab="",main="")
      title(main=list(parse.unit.xlab(fm[k,"unit"],fm[k,"feature"]),cex=1.5))
      for(a in 1:length(uspecies)){
        xx <- sort(pc[[a]])
        lines(xx,c(1:length(xx))/length(xx),col=col.cum[a],lwd=2)
      }
      if(legend){
        legend("bottomright",col=col.cum,lty=1,lwd=2,legend=plot_labels,box.lty=0)
      }
    }
  }
  ymax <- ceiling(length(plot_labels)/ncol_legend)

  plot(0,0,col=0,ylim=c(0,ymax+1),xlim=c(0,ncol_legend*2),main="",axes=FALSE,xlab="",ylab="")
  start <- 1;
  end <- ymax;
  for(k in 1:ncol_legend){

    legend(x=(k-1)*2,y=ymax,legend=plot_labels[start:end],lty=1,col=col.cum[start:end],box.lty=0,cex=1,lwd=2)
    start <- end + 1;
    end <- end + ymax;
    if(end > length(plot_labels)){
      end <- length(plot_labels)
    }
  }
  dev.off()

}

#' @title Plot scatter plots of feature values.
#'
#' @description This function plots scatter plots of all features where the scatter plots of each feature versus all other features are plotted on a separate page of a pdf file. Plots are saved to the \code{scatter_plots} folder in the \code{output} folder (@seealso \code{\link{setOutputFolder}}).
#'
#' @param data feature data containing list generated by \code{\link{readFeatureData}} and \code{\link{filterOutSmallCells}}.
#' @param type of the data can be
#' \describe{
#'     \item{raw}{all measured cell areas. Default.}
#'     \item{filtered}{cell areas above a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#'     \item{small}{cell areas below a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#' }
#' @param legend \code{TRUE} or \code{FALSE} depending on whether the legend should be plotted in the upper right corner of the plot or once on a separate page. Default is \code{legend=FALSE}.
#' @param ncol_legend number of columns to arrange to content of the legend. Default is \code{ncol_legend=1}.
#'
#' @export
#'
plotScatterplots <- function(data=data,type="raw",legend=FALSE,ncol_legend=1){
  #pc.mat,species,features.map,outdir,base_file_name,col.sca){

  outdir <- paste0(pkg.globals$folder.results,pkg.globals$fsep,"scatter_plots");
  base_file_name <- paste0(type,"_data");

  fsep <- pkg.globals$fsep;
  features.map <- feature.globals$fmap;

  tmpf <- checkType(data,type);
  species <- tmpf$species;
  pc.mat <- tmpf$pc.mat;
  rois <- tmpf$rois;

  plot_labels <- desc.globals$plot_labels[order(desc.globals$plot_order)]
  plot_font <- desc.globals$plot_font[order(desc.globals$plot_order)]
  col.sca <- desc.globals$plot_colors[order(desc.globals$plot_order)]

  uspecies <- pkg.globals$dirs[order(desc.globals$plot_order)]

  for(j in 1:nrow(features.map)){
    x <- pc.mat[,features.map[j,"order_column"]]
    xlab <- parse.unit.xlab(features.map[j,"unit"],features.map[j,"feature"])
    xlim <- range(x,na.rm=TRUE)
    pdf(file=paste0(outdir,fsep,base_file_name,"_scatterplots_",features.map[j,"feature"],".pdf"),width=4,height=4,family="Arial",useDingbats=FALSE)
    for(k in 1:nrow(features.map)){
      y <- pc.mat[,features.map[k,"order_column"]]
      ylim <- range(y,na.rm=TRUE)
      plot(x,y,xlim=xlim,ylim=ylim,xlab=xlab,ylab=parse.unit.xlab(features.map[k,"unit"],features.map[k,"feature"]),pch=16,col=col.sca[species])
      if(legend){
        legend(x=min(x),y=max(y),col=col.sca,pch=16,legend=uspecies,box.lty=0,xpd=NA,yjust=0,xjust=0.1,horiz=TRUE)
      }
    }

    ymax <- ceiling(length(plot_labels)/ncol_legend)

    plot(0,0,col=0,ylim=c(0,ymax+1),xlim=c(0,ncol_legend*2),main="",axes=FALSE,xlab="",ylab="")
    start <- 1;
    end <- ymax;
    for(k in 1:ncol_legend){
      #print(start)
      #print(end)

      legend(x=(k-1)*2,y=ymax,legend=plot_labels[start:end],pch=16,col=col.sca[start:end],box.lty=0,cex=1)
      start <- end + 1;
      end <- end + ymax;
      if(end > length(plot_labels)){
        end <- length(plot_labels)
      }
    }

    dev.off()
  }



}

