
#' @title Perform Kruskal-Wallis-test and post-hoc Dunn-tests.
#'
#' @description This function performs Kruskal-Wallis-tests and post-hoc Dunn-tests for each feature and defined groups. Resulting p values are saved in two separate files, one for the Kruskal-Wallis-tests and one for the Dunn-tests. p values from the Dunn-tests are additionally visualized in a heatmap. Both, lists of pvalues and heatmap are saved to the \code{kw_dunn} folder in the \code{output} folder (@seealso \code{\link{setOutputFolder}}).
#'
#' @param data feature data containing list generated by \code{\link{readFeatureData}} and \code{\link{filterOutSmallCells}}.
#' @param type of the data can be
#' \describe{
#'     \item{raw}{all measured cell areas. Default.}
#'     \item{filtered}{cell areas above a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#'     \item{small}{cell areas below a threshold for small cells (@seealso \code{\link{setSizeThresholds}})}
#' }
#' @param omi.b defines the space at the bottom of the heatmap used for printing the labels. Default is \code{omi.b=1}.
#'
#' @export
#'
kwDunnTest <- function(data=data,type="filtered",omi.b=1){
#mat,features,species,outdir,base_file_name,omi.b=1,plot_labs=plot.labels

  outdir <- paste0(pkg.globals$folder.results,pkg.globals$fsep,"kw_dunn");
  base_file_name <- paste0(type,"_data");

  fsep <- pkg.globals$fsep;
  features <- feature.globals$feat;

  # use checkType from functionsPlots.r
  tmpf <- checkType(data,type);
  species <- tmpf$species;
  mat <- tmpf$pc.mat;
  rois <- tmpf$rois;

  # length uspecies
  # min 2 fuer KW test
  # min 3 fuer Dunn test
  uspecies <- pkg.globals$dirs[order(desc.globals$plot_order)]
  plot_labs <- desc.globals$plot_labels[order(desc.globals$plot_order)]

  #print(uspecies)
  sf <- factor(species,levels=uspecies)
  dt.mat <- matrix(NA,nrow=0,ncol=(length(uspecies) * (length(uspecies) - 1))/2)
  kw.stat <- vector(length=0)
  for(fg in names(features)){
    fm <- features[[fg]]
    tmp <- matrix(NA,nrow=nrow(fm),ncol=ncol(dt.mat))
    #		colnames(tmp) <- colnames(dt.mat)
    rownames(tmp) <- rownames(fm)
    for(k in 1:nrow(fm)){
      d <- mat[,fm[k,"order_column"]]
      idx.d <- !is.na(d)
      kw.stat.tmp <- kruskal.test(x=d[idx.d],g=sf[idx.d])$p.value
      kw.stat <- c(kw.stat,kw.stat.tmp)
      if(length(uspecies) > 2){
        dt <- dunn.test(x=d[idx.d],g=sf[idx.d],method="bh",table=FALSE,list=FALSE,alpha=0.05)
        clabels <- dt$comparisons
        tmp[k,] <- dt$P.adjusted
      }else{
        tmp[k,] <- kw.stat.tmp
        clabels <- paste(uspecies,collapse=" - ")
      }
    }
    dt.mat <- rbind(dt.mat,tmp)
  }
  #print(clabels)
  # replace labels
  col_labels <- clabels
  for(j in 1:length(col_labels)){
    lab <- strsplit(col_labels[j]," - ")[[1]]
    col_labels[j] <- paste0(plot_labs[lab[1]]," - ",plot_labs[lab[2]])
  }

  colnames(dt.mat) <- col_labels
  names(kw.stat) <- rownames(dt.mat)

  write.csv(kw.stat,file=paste0(outdir,fsep,base_file_name,"_kruskal_wallis_test.csv"))
  write.csv(dt.mat,file=paste0(outdir,fsep,base_file_name,"_dunn_test.csv"))
  dt.mat.note <- matrix("",nrow=nrow(dt.mat),ncol=ncol(dt.mat))
  dt.mat.note[dt.mat < 0.05] <- "*"
  dt.mat.note[dt.mat < 0.01] <- "**"
  dt.mat.note[dt.mat < 0.001] <- "***"
  dummy <<- function(cnames=colnames(dt.mat)){
    mtext(cnames,side=1,line=1,at=1:length(cnames),cex=0.8,las=2)
  }
  #breaks <- seq(0,1,by=0.01)
  breaks <- 2^seq(log2(0.001),log2(0.5),by=0.1)

  col <- c("black","magenta","darkgreen","blue")
  colRow <- c(rep(col[1],11),rep(col[2],3),rep(col[3],4),rep(col[4],9))
  if(length(uspecies) > 1){

    #3+ncol(dt.mat)
    width <- 9
    if((3 + ncol(dt.mat)) > width){
      width <- 3 + ncol(dt.mat)
    }

    if(length(uspecies) > 2){
      height <- 11;#9
      png(file=paste0(outdir,fsep,base_file_name,"_dunn_test_heatmap_adjusted_pvalues.png"),width=width,height=height,res=300,units="in",family="Arial")
      par(omi=c(omi.b,0.5,0.5,2))
      heatmap.2(dt.mat,main="Benjamini-Hochberg-adjusted\np-values after Dunn's test",trace="none",breaks=breaks,symkey=FALSE,Rowv=FALSE,Colv=FALSE,dendrogram="none",col=colorRampPalette(c("gold","royalblue")),na.col="lightgrey",lwid=c(0.2,0.8),lhei=c(0.2,0.8),scale="none",colsep=1:(ncol(dt.mat)-1),rowsep=c(11,14,18),colRow=colRow,cellnote=dt.mat.note,notecol="black",add.expr=dummy(),key.xlab="BH-adjusted p value",key.title=NA,key.ylab="Count",labCol=c(c(rep("",ncol(dt.mat)))))
      dev.off()


      pdf(file=paste0(outdir,fsep,base_file_name,"_dunn_test_heatmap_adjusted_pvalues.pdf"),width=width,height=height,useDingbats=FALSE,family="Arial")
      par(omi=c(omi.b,0.5,0.5,2))
      heatmap.2(dt.mat,main="Benjamini-Hochberg-adjusted\np-values after Dunn's test",trace="none",breaks=breaks,symkey=FALSE,Rowv=FALSE,Colv=FALSE,dendrogram="none",col=colorRampPalette(c("gold","royalblue")),na.col="lightgrey",lwid=c(0.2,0.8),lhei=c(0.2,0.8),scale="none",colsep=1:(ncol(dt.mat)-1),rowsep=c(11,14,18),colRow=colRow,cellnote=dt.mat.note,notecol="black",add.expr=dummy(),key.xlab="BH-adjusted p value",key.title=NA,key.ylab="Count",labCol=c(c(rep("",ncol(dt.mat)))))
      dev.off()
    }
  }
  #return(list(kw.test=kw.stat,dunn.test=dt.mat))

}
